{*******************************************************}
{                                                       }
{       eXpert Desktop Page                             }
{                                                       }
{       Copyright (c) 1996,97 Sergey Orlik              }
{       - product manager of Borland Russia             }
{                                                       }
{*******************************************************}
unit XDPage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, ComCtrls;

type
  TxdPageClass = class of TxdPage;
  TxdPage = class(TPanel)
  private
    FOnCreate: TNotifyEvent;
    FOnDestroy: TNotifyEvent;
  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    property Align;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    property BevelOuter default bvNone;
    property OnCreate: TNotifyEvent read FOnCreate write FOnCreate;
    property OnDestroy: TNotifyEvent read FOnDestroy write FOnDestroy;
  end;

procedure RegisterXDPage(APageClass:TxdPageClass;var APage);
procedure UnRegisterXDPage(var APage);

//================================================================================
implementation
//================================================================================

uses
  XDesk;

resourcestring
  sResNotFound = 'Resource for eXpert Desktop Page is not found.';

constructor TxdPage.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  if AOwner is TWinControl then
    Parent:=AOwner as TWinControl;
  if ClassType<>TxdPage then
  begin
    if not InitInheritedComponent(Self, TxdPage) then
      raise EResNotFound.CreateFmt(sResNotFound, [ClassName]);
  end;
  try
    if Assigned(FOnCreate) then FOnCreate(Self);
  except
    Application.HandleException(Self);
  end;
  BevelOuter:=bvNone;
end;

destructor TxdPage.Destroy;
begin
  if Assigned(FOnDestroy) then
  try
    FOnDestroy(Self);
  except
    Application.HandleException(Self);
  end;
  inherited Destroy;
end;

procedure TXDPage.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
  OwnedComponent: TComponent;
begin
  inherited GetChildren(Proc, Root);
  if Root = Self then
  begin
    for I := 0 to ComponentCount - 1 do
    begin
      OwnedComponent := Components[I];
      if not OwnedComponent.HasParent then Proc(OwnedComponent);
    end;
  end;
end;

procedure TxdPage.SetChildOrder(Child: TComponent; Order: Integer);
var
  I, J: Integer;
begin
  if Child is TControl then
    inherited SetChildOrder(Child, Order)
  else
  begin
    Dec(Order, ControlCount);
    J := -1;
    for I := 0 to ComponentCount - 1 do
      if not Components[I].HasParent then
      begin
        Inc(J);
        if J = Order then
        begin
          Child.ComponentIndex := I;
          Exit;
        end;
      end;
  end;
end;

//================================================================================
// Register XDPage

resourcestring
  sRegNewPageError = 'New eXpert Desktop Page registration error';

procedure ErrorRegNewPage;
begin
  raise Exception.Create(sRegNewPageError);
end;

procedure RegisterXDPage(APageClass:TxdPageClass;var APage);
var
  Page : TTabSheet;
  Instance: TxdPage;
begin
  if ExpertDesktop=nil then Exit;
  Page:=TTabSheet.Create(ExpertDesktop);
  Page.Parent:=ExpertDesktop.Pages;
  Page.PageControl:=ExpertDesktop.Pages;
  Instance := TxdPage(APageClass.NewInstance);
  TxdPage(APage):=Instance;
  try
    Instance.Create(Page);
//    Instance.Parent:=Page;
    Instance.Align:=alClient;
    Instance.Name:='Page'+IntToStr(ExpertDesktop.Pages.PageCount+1);
    if Instance.Caption<>EmptyStr then
      Page.Caption:=Instance.Caption
    else
      Page.Caption:=APageClass.ClassName;
    Instance.Caption:=EmptyStr;
  except
    TxdPage(APage):=nil;
    Instance.Free;
    Page.Free;
    ErrorRegNewPage;
  end;
end;

procedure UnRegisterXDPage(var APage);
var
  Page : TTabSheet;
begin
  if TxdPage(APage)<>nil then
  begin
    Page:=TTabSheet(TxdPage(APage).Owner);
    TxdPage(APage).Free;
    TxdPage(APage):=nil;
    Page.Free;
  end;
end;

end.


